'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _colors3 = require('pad/lib/colors');

var _colors4 = _interopRequireDefault(_colors3);

var _progress = require('progress');

var _progress2 = _interopRequireDefault(_progress);

var _quizGenerator = require('../quiz-generator.js');

var _quizGenerator2 = _interopRequireDefault(_quizGenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; } /* eslint-disable no-console */


const showUsage = () => {
  console.log(`word-quiz-generator coverage --help
word-quiz-generator coverage --material=<path> --sources=<paths> [--lang]
                             [--display-details] [--display-lemmas]
                             [--sentenceSeparator=<RegExp>] [--clauseRegExp=<RegExp>]
                             [--wordRegExp=<RegExp>] [--wordBoundaryRegExp=<RegExp>]
                             [--abbrRegExp=<RegExp>]

Measure the coverage of words/phrases in the given material against the given sources.

-h, --help
    Show this usage.
-m, --material=<path>
    Path to a material.
-s, --sources=<paths>
    Comma-separated path strings to sources.
-l, --lang=<lang>
    IETF langage tag in which the material are written.
    This determines how to extract a word/phrase or sentence from a text.
    If you need more precise control over the extraction algorithm,
    please use '--sentenceSeparator', '--clauseRegExp', '--wordRegExp',
    '--wordBoundaryRegExp', and/or '--abbrRegExp' to override.
    Default: 'en' (English)
-d, --display-details
    Show more information about the results, such as the list of
    uncovered words/phrases, suggestions to improve the coverage, etc.
--display-lemmas
    Show lemmas of phrases in the suggestions discribed above.
    This option is separated from '--display-details'
    because it is extremely slow when trying to display lemmas of English using TreeTagger :/
--sentenceSeparator=<RegExp>
    Regular expression representing a sentence separator.
--clauseRegExp=<RegExp>
    Regular expression representing a clause.
--wordRegExp=<RegExp>
    Regular expression representing a word.
--wordBoundaryRegExp=<RegExp>
    Regular expression representing a word boundary.
--abbrRegExp=<RegExp>
    Regular expression representing an abbreviation mark.`);
};

exports.default = (() => {
  var ref = _asyncToGenerator(function* (args) {
    const argv = (0, _minimist2.default)(args, {
      string: ['material', 'sources', 'lang', 'sentenceSeparator', 'clauseRegExp', 'wordRegExp', 'wordBoundaryRegExp', 'abbrRegExp'],
      boolean: ['display-details', 'display-lemmas', 'help'],
      alias: {
        m: 'material',
        s: 'sources',
        l: 'lang',
        d: 'display-details',
        h: 'help'
      },
      default: {
        lang: 'en'
      }
    });

    if (argv.help) {
      showUsage();
      process.exit(0);
    }

    if (!argv.material || !argv.sources) {
      showUsage();
      process.exit(1);
    }

    const material = argv.material;
    const sources = argv.sources;
    const lang = argv.lang;
    const sentenceSeparator = argv.sentenceSeparator && new RegExp(argv.sentenceSeparator, 'g');
    const clauseRegExp = argv.clauseRegExp && new RegExp(argv.clauseRegExp, 'g');
    const wordRegExp = argv.wordRegExp && new RegExp(argv.wordRegExp, 'g');
    const wordBoundaryRegExp = argv.wordBoundaryRegExp && new RegExp(argv.wordBoundaryRegExp);
    const abbrRegExp = argv.abbrRegExp && new RegExp(argv.abbrRegExp, 'g');
    const generator = new _quizGenerator2.default({
      material, sources, lang, sentenceSeparator, clauseRegExp, wordRegExp,
      wordBoundaryRegExp, abbrRegExp
    });

    yield generator.init();

    const phrases = generator._phrases;
    const total = phrases.length;
    const uncovered = [];
    const progress = new _progress2.default('[:bar] :percent :etas', {
      complete: '=',
      incomplete: ' ',
      width: 20,
      total: phrases.length
    });

    while (phrases.length) {
      const phrase = phrases.shift();
      const question = yield generator.question(phrase);

      if (!question) {
        uncovered.push(phrase);
      }

      progress.tick();
    }

    const coverage = (total - uncovered.length) * 100 / total;

    console.log(_colors2.default.bold(`Coverage: ${ coverage.toFixed(2) }%`));

    if (argv['display-details']) {
      console.log(_colors2.default.bold('\nUncovered Phrases'));

      console.log(`
-----
Legend: ${ '#'.grey } phrase ${ 'answer'.grey } ${ 'reference'.green } ${ 'lemmas'.red }

- A reference name is shown if a phrase is found in an original version of source,
  which means you may somehow make the phrase detectable from the generator,
  such as by turning them into their lemmas.

- Lemmas of a phrase are shown if they are different from an original phrase,
  which indicates the phrase is shown here because they are stored in natural form
  and not recognizable to the generator.
  ${ 'Lemmas are shown only when you run with \'--display-lemmas\' option.'.bold }
-----
`);

      const lemmatizer = require(`../lemmatizers/${ argv.lang }.js`).default;
      const findWithinOriginals = (() => {
        var ref = _asyncToGenerator(function* (phrase) {
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = generator._sources[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              const source = _step.value;

              const text = yield source.getText();

              if (phrase.split('|').some(function (exp) {
                return text.includes(exp);
              })) {
                return source;
              }
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          return null;
        });

        return function findWithinOriginals(_x2) {
          return ref.apply(this, arguments);
        };
      })();

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = uncovered[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          const phrase = _step2.value;

          const source = yield findWithinOriginals(phrase.phrase);
          const message = [];

          message.push((0, _colors4.default)(2, _colors2.default.grey(phrase.section)));
          message.push((0, _colors4.default)(phrase.phrase, 20));
          message.push((0, _colors4.default)(_colors2.default.grey(phrase.answer), 20));

          if (source) {
            const ref = yield source.getReference();
            message.push((0, _colors4.default)(_colors2.default.green(ref), 10));
          } else {
            message.push((0, _colors4.default)('', 10));
          }

          if (argv['display-lemmas']) {
            const lemma = yield lemmatizer(phrase.phrase);

            if (lemma !== phrase.phrase) {
              message.push(_colors2.default.red(lemma));
            }
          }

          console.log(message.join(' '));
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
  });

  return function (_x) {
    return ref.apply(this, arguments);
  };
})();
//# sourceMappingURL=coverage.js.map