'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

let clean = (() => {
  var ref = _asyncToGenerator(function* (dir) {
    const files = yield (0, _utils.fetchFileList)(dir, /\.(?:preprocessed|lemmatized)$/);

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const _path = _step.value;

        yield _fsExtraPromise2.default.removeAsync(_path);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  });

  return function clean(_x) {
    return ref.apply(this, arguments);
  };
})();

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _minimist = require('minimist');

var _minimist2 = _interopRequireDefault(_minimist);

var _fsExtraPromise = require('fs-extra-promise');

var _fsExtraPromise2 = _interopRequireDefault(_fsExtraPromise);

var _source = require('../source.js');

var _source2 = _interopRequireDefault(_source);

var _utils = require('../utils.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; } /* eslint-disable no-console */


const showUsage = () => {
  console.log(`word-quiz-generator make --help
word-quiz-generator make --src=<path> --lang=<lang>
word-quiz-generator make --src=<path> [--preprocessor=<path>] [--lemmatizer=<path>]

Generate preprocessed and lemmatized texts from the given sources.

-h, --help
    Show this usage.
-s, --src=<paths>
    Comma-separated path strings to be processed.
-l, --lang=<lang>
    IETF langage tag in which source texts are written.
    This determines which built-in preprocesser and lemmatizer should be used.
    If you want to use your custom ones, please use '--preprocessor' and '--lemmatizer' options.
    Default: 'en' (English)
--preprocessor=<path>
  Path to a custom preprocessor.
--lemmatizer=<path>
  Path to a custom lemmatizer.`);
};

function getPreprocessor(argv) {
  try {
    if (argv.preprocessor) {
      const preprocessor = require(argv.preprocessor);
      return preprocessor.default || preprocessor;
    } else if (argv.lang) {
      return require(`../preprocessors/${ argv.lang }.js`).default;
    }
  } catch (err) {
    console.error('Unable to load the preprocessor: %c', err.message);
  }

  return null;
}

function getLemmatizer(argv) {
  try {
    if (argv.lemmatizer) {
      const lemmatizer = require(argv.lemmatizer);
      return lemmatizer.default || lemmatizer;
    } else if (argv.lang) {
      return require(`../lemmatizers/${ argv.lang }.js`).default;
    }
  } catch (err) {
    console.error('Unable to load the lemmatizer: %c', err.message);
  }

  return null;
}

exports.default = (() => {
  var ref = _asyncToGenerator(function* (args) {
    const argv = (0, _minimist2.default)(args, {
      string: ['src', 'lang', 'preprocessor', 'lemmatizer'],
      boolean: ['help'],
      alias: {
        s: 'src',
        l: 'lang',
        h: 'help'
      },
      default: {
        lang: 'en',
        help: false
      }
    });

    if (argv.help) {
      showUsage();
      process.exit(0);
    }

    if (!argv.src) {
      console.error('ERR: No src is specified.');
      showUsage();
      process.exit(1);
    }

    console.log(`Src: ${ _path3.default.resolve(argv.src) }`);

    yield clean(argv.src);

    const preprocessor = getPreprocessor(argv);
    const lemmatizer = getLemmatizer(argv);
    const files = yield (0, _utils.fetchFileList)(argv.src, /\.txt$/);

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = files[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        const _path = _step2.value;

        console.log(_path);

        try {
          const source = new _source2.default(_path);

          if (preprocessor) {
            console.log('Preprocessing...');
            yield source.preprocess(preprocessor);
          }

          if (lemmatizer) {
            console.log('Lemmatizing...');
            yield source.lemmatize(lemmatizer);
          }
        } catch (ex) {
          console.error(ex.stack);
        }
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2.return) {
          _iterator2.return();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }
  });

  return function (_x2) {
    return ref.apply(this, arguments);
  };
})();
//# sourceMappingURL=make.js.map