'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _mecab = require('../mecab.js');

var _mecab2 = _interopRequireDefault(_mecab);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * Lemmatizer for Classical Japanese using MeCab
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          */


const MECAB_WORD = 0;
const MECAB_LEMMA = 13;
const mecabHome = _path2.default.join(__dirname, '..', '..', 'vendor', 'mecab');
const mecabPath = _path2.default.join(mecabHome, 'mecab', 'bin', 'mecab');
const rcPath = _path2.default.join(mecabHome, 'unidic-ojp', '.mecabrc-ojp');
const mecab = new _mecab2.default({ command: `${ mecabPath } --rcfile=${ rcPath }` });

exports.default = (() => {
  var ref = _asyncToGenerator(function* (text) {
    const chunks = text.replace(/ /g, '').split(/\n/).map(function (block) {
      return block.split(/。/);
    });
    const lemmatizedChunks = [];

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = chunks[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const chunk = _step.value;

        const results = [];

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = chunk[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            const sentence = _step2.value;

            const parsed = yield mecab.parse(sentence);
            const lemmatized = parsed.map(function (word) {
              return(
                // '*' means there is no lemma data for this word.
                word[MECAB_LEMMA] && word[MECAB_LEMMA] !== '*' ? word[MECAB_LEMMA] : word[MECAB_WORD]
              );
            }).join(' ');

            results.push(lemmatized);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        lemmatizedChunks.push(results.join(' 。 '));
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return lemmatizedChunks.join('\n');
  });

  return function (_x) {
    return ref.apply(this, arguments);
  };
})();
//# sourceMappingURL=ojp.js.map