'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

/**
 * Apply transformer to every tokens extracted from a text by Mecab
 * @param {string} text a target text
 * @param {Function<Promise<string>>} transformer
 * @return {string} transformed text
 */

let transform = (() => {
  var ref = _asyncToGenerator(function* (text, transformer) {
    const chunks = text.split(/\n/).map(function (block) {
      return block.split(/。/);
    });
    const results = [];

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = chunks[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const chunk = _step.value;

        const result = [];

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = chunk[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            const sentence = _step2.value;

            const parsed = yield mecab.parse(sentence);
            const transformed = parsed.map(transformer);

            result.push(transformed.join(''));
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        results.push(result.join('。'));
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return results.join('\n');
  });

  return function transform(_x, _x2) {
    return ref.apply(this, arguments);
  };
})();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _xregexp = require('xregexp');

var _xregexp2 = _interopRequireDefault(_xregexp);

var _mecab = require('../mecab.js');

var _mecab2 = _interopRequireDefault(_mecab);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

const MECAB_WORD = 0;
const mecabHome = _path2.default.join(__dirname, '..', '..', 'vendor', 'mecab');
const mecabPath = _path2.default.join(mecabHome, 'mecab', 'bin', 'mecab');
const rcPath = _path2.default.join(mecabHome, 'unidic-ojp', '.mecabrc-ojp');
const mecab = new _mecab2.default({ command: `${ mecabPath } --rcfile=${ rcPath }` });

function toVoicedChar(char) {
  const unvoiced = 'かきくけこさしすせそたちつてとはひふへほ';
  const voiced = 'がぎぐげござじずぜぞだじづでどばびぶべぼ';
  const index = unvoiced.indexOf(char);

  return index < 0 ? char : voiced[index];
}

function removeAnnotations(text) {
  /**
   * Remove a string between `left` and `right` using recursive RegExp.
   * @param {string} str a target string
   * @param {string} left left delimiter
   * @param {string} right right delimiter
   * @return {string}
   */
  const removeBetween = (str, left, right) => {
    const outsides = _xregexp2.default.matchRecursive(str, left, right, 'g', {
      valueNames: ['outside', null, null, null]
    });

    return outsides.map(outside => outside.value).join('');
  };

  // ---- Replace full-width brackets with their half-width ones.
  let _text = text.replace(/（/g, '(').replace(/）/g, ')').replace(/［/g, '[').replace(/］/g, ']').replace(/｛/g, '{').replace(/｝/g, '}');

  // --- Remove annotations.
  const brackets = [['\\(', '\\)'], ['\\[', '\\]'], ['\\{', '\\}'], ['〔', '〕'], ['【', '】']];

  brackets.forEach(pair => {
    try {
      _text = removeBetween(_text, ...pair);
    } catch (err) {
      // Fallback
      // if brackets are not properly paired,
      // try to remove the most appropriate parts of the text.

      var _pair = _slicedToArray(pair, 2);

      const left = _pair[0];
      const right = _pair[1];


      _text = _text.replace(new RegExp(`${ left }.*?${ right }`, 'g'), '');
      _text = _text.replace(new RegExp(`(?:${ left }|${ right })`, 'g'), '');
    }
  });

  return _text;
}

exports.default = (() => {
  var ref = _asyncToGenerator(function* (text) {
    // ---- Remove annotations
    const textWithoutAnnotations = removeAnnotations(text);

    const emended = textWithoutAnnotations
    // --- Replace substitutions of repeat marks with their unicode characters.
    .replace(/／〃＼/g, '〲').replace(/＼〃／/g, '〲').replace(/／＼/g, '〱').replace(/＼／/g, '〱')

    // --- Un-odorijify (single character repetition)
    .replace(/(.)ゝ/g, '$1$1').replace(/(.)ゞ/g, function (__, prev) {
      return `${ prev }${ toVoicedChar(prev) }`;
    })

    // --- Remove brackets to make a question sentence tidy.

    // Replace some half-width signs with their full-width ones.
    .replace(/｢/g, '「').replace(/｣/g, '」')

    // Remove brackets
    .replace(/(?:「|」|『|』)/g, '')

    // --- Remove full-width spaces
    // eslint-disable-next-line no-irregular-whitespace
    .replace(/　/g, '');

    // --- Un-odorijify (multi characters repetition)
    const unodorijified = yield transform(emended, function (table, i, parsed) {
      const word = table[MECAB_WORD];

      // un-odorijify (unvoiced)
      if (word.startsWith('〱')) {
        const prev = parsed[i - 1][MECAB_WORD];

        if (prev.length >= 2) {
          return word.replace(/〱/, prev);
        }
      }

      // Un-odorijify (voiced)
      if (word.startsWith('〲')) {
        const prev = parsed[i - 1][MECAB_WORD];

        if (prev.length >= 2) {
          return word.replace(/〲/, `${ toVoicedChar(prev[0]) }${ prev.substring(1) }`);
        }
      }

      return word;
    });

    // --- Wakachigaki
    // Separate each words with a half-width space (wakachigaki)
    // to enable QuizGenerator to detect word boundaries easily.
    const wakachigaki = yield transform(unodorijified, function (table) {
      return `${ table[MECAB_WORD] } `;
    });

    // --- Remove trailing spaces
    return wakachigaki.replace(/ $/mg, '');
  });

  return function (_x3) {
    return ref.apply(this, arguments);
  };
})();
//# sourceMappingURL=ojp.js.map