'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _regexpPresets = require('./constants/regexp-presets');

var _regexpPresets2 = _interopRequireDefault(_regexpPresets);

var _wordDivider = require('./constants/word-divider');

var _wordDivider2 = _interopRequireDefault(_wordDivider);

var _source = require('./source.js');

var _source2 = _interopRequireDefault(_source);

var _utils = require('./utils.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

const log = (0, _debug2.default)('word-quiz-generator');

/**
 * Generate a quiz (a set of questions)
 * randomly selecting some phrases from the given material file
 * and randomly selecting question sources from the given sources.
 * NOTE: You have to run `init()` before using any other methods!
 */

let QuizGenerator = function () {

  /**
   * @param {string} material Path to a CSV-formatted material file.
   * @param {string} sources Pathes to source text files or directries.
   * @param {string} [lang='en']
   *   IETF language tag in which the material written.
   *   `lang` is used for determining which RegExp preset should be used.
   *   If not specified or not found in the presets, the 'en' preset will be used.
   *   If you need to override each RegExps, please use individual parameters below.
   * @param {RegExp} [sentenceSeparator] Regular expression representing a sentence separator.
   * @param {RegExp} [clauseRegExp] Regular expression representing a clause.
   * @param {RegExp} [wordRegExp] Regular expression representing a word.
   * @param {RegExp} [wordBoundaryRegExp] Regular expression representing a word boundary.
   * @param {RegExp} [abbrRegExp] Regular expression representing an abbreviation mark.
   */

  function QuizGenerator(_ref) {
    let material = _ref.material;
    let sources = _ref.sources;
    var _ref$lang = _ref.lang;
    let lang = _ref$lang === undefined ? 'en' : _ref$lang;
    let sentenceSeparator = _ref.sentenceSeparator;
    let clauseRegExp = _ref.clauseRegExp;
    let wordRegExp = _ref.wordRegExp;
    let wordBoundaryRegExp = _ref.wordBoundaryRegExp;
    let abbrRegExp = _ref.abbrRegExp;

    _classCallCheck(this, QuizGenerator);

    this._material = material;
    this._sources = sources;
    this._lang = lang;
    this._sentenceSeparator = sentenceSeparator;
    this._clauseRegExp = clauseRegExp;
    this._wordRegExp = wordRegExp;
    this._wordBoundaryRegExp = wordBoundaryRegExp;
    this._abbrRegExp = abbrRegExp;
  }

  /**
   * Load and parse the files of phrases and sources.
   */


  _createClass(QuizGenerator, [{
    key: 'init',
    value: (() => {
      var ref = _asyncToGenerator(function* () {
        this._phrases = yield (0, _utils.parseMaterial)(this._material);
        this._sources = yield (0, _utils.fetchFileList)(this._sources, /\.txt$/);
        this._sources = this._sources.map(function (path) {
          return new _source2.default(path);
        });
        this._loadRegExpPreset();
      });

      function init() {
        return ref.apply(this, arguments);
      }

      return init;
    })()
  }, {
    key: '_loadRegExpPreset',
    value: function _loadRegExpPreset() {
      const preset = _regexpPresets2.default[this._lang] || _regexpPresets2.default.en;

      this._sentenceSeparator = this._sentenceSeparator || preset.sentenceSeparator;
      this._clauseRegExp = this._clauseRegExp || preset.clauseRegExp;
      this._wordRegExp = this._wordRegExp || preset.wordRegExp;
      this._wordBoundaryRegExp = this._wordBoundaryRegExp || preset.wordBoundaryRegExp;
      this._abbrRegExp = this._abbrRegExp || preset.abbrRegExp;
    }

    /**
     * Generate a quiz.
     * @param {string} sections "-" separated string
     *                          representing a scope of the quiz.
     *                          e.g. 5-10
     * @param {number} size # of questions in the quiz.
     * @return {Array<Question>} Generated quiz.
     */

  }, {
    key: 'quiz',
    value: (() => {
      var ref = _asyncToGenerator(function* (_ref2) {
        let sections = _ref2.sections;
        let size = _ref2.size;

        var _sections$split$map = sections.split('-').map(function (num) {
          return Number(num);
        });

        var _sections$split$map2 = _slicedToArray(_sections$split$map, 2);

        var _sections$split$map2$ = _sections$split$map2[0];
        const min = _sections$split$map2$ === undefined ? 0 : _sections$split$map2$;
        const max = _sections$split$map2[1];

        const phrasesToTest = _lodash2.default.shuffle(this._phrases.filter(function (phrase) {
          return _lodash2.default.inRange(phrase.section, min, (max || min) + 1);
        }));
        const quiz = [];

        while (quiz.length < size && phrasesToTest.length > 0) {
          const phrase = phrasesToTest.pop();
          const question = yield this.question(phrase);

          if (question) {
            quiz.push(question);
          }
        }

        return quiz;
      });

      function quiz(_x) {
        return ref.apply(this, arguments);
      }

      return quiz;
    })()

    /**
     * Generate a question using the given phrase.
     * @param {{phrase, answer, reference}} phrase
     * @return {Question}
     */

  }, {
    key: 'question',
    value: (() => {
      var ref = _asyncToGenerator(function* (phrase) {
        const sources = _lodash2.default.shuffle(this._sources);

        while (sources.length > 0) {
          const src = sources.pop();
          const question = yield this.generateQuestionFromSource({ phrase, src });

          if (!question) {
            continue;
          }

          return question;
        }

        return null;
      });

      function question(_x2) {
        return ref.apply(this, arguments);
      }

      return question;
    })()

    /**
     * Generate a question using the given phrase and source.
     * @param {{phrase, answer, reference}} phrase
     * @param {Source} src
     * @return {Question}
     */

  }, {
    key: 'generateQuestionFromSource',
    value: (() => {
      var ref = _asyncToGenerator(function* (_ref3) {
        let phrase = _ref3.phrase;
        let src = _ref3.src;

        const result = yield this.selectSentence({ phrase: phrase.phrase, src });

        if (!result) {
          return null;
        }

        const sentenceIndex = result.sentenceIndex;
        const wordIndexes = result.wordIndexes;

        const text = yield src.preprocessed.getText();
        const reference = yield src.preprocessed.getReference();
        const sentence = this.getSentenceAt({ index: sentenceIndex, text });

        /* istanbul ignore if */
        if (!sentence) {
          return null;
        }

        const body = this.parseQuestionSentence({ sentence, wordIndexes });

        return {
          body,
          answer: phrase.answer,
          reference,
          phrase: phrase.phrase
        };
      });

      function generateQuestionFromSource(_x3) {
        return ref.apply(this, arguments);
      }

      return generateQuestionFromSource;
    })()

    /**
     * @param {string} original (non-lemmatized) sentence
     * @param {Array<number>} wordIndexes index numbers representing which word is a part of question.
     * @return {Array<{ text: string, isQuestionPart: bool, isMark: bool, isDivider: bool }>}
     */

  }, {
    key: 'parseQuestionSentence',
    value: function parseQuestionSentence(_ref4) {
      let sentence = _ref4.sentence;
      let wordIndexes = _ref4.wordIndexes;

      // Here we assume word divider is space.
      // If you are dealing with a language it is not the case (e.g. Japanese),
      // firstly you have to convert a text using a preprocessor.
      const tokenRegExp = new RegExp(`(${ this._wordRegExp.source })(\\s*)`, 'g');
      const tokens = [];
      const divider = _lodash2.default.findKey(_wordDivider2.default, regs => regs.some(reg => reg.test(this._lang)));
      let currentWordIndex = 0;
      let prevLastIndex = 0;
      let matched;

      // eslint-disable-next-line no-cond-assign
      while (matched = tokenRegExp.exec(sentence)) {
        var _matched = matched;

        var _matched2 = _slicedToArray(_matched, 3);

        const token = _matched2[1];
        const hasDivider = _matched2[2];

        const punctuation = sentence.substring(prevLastIndex, matched.index);
        const isQuestionWord = wordIndexes[0] === currentWordIndex;
        const isSuccessive = isQuestionWord && wordIndexes[0] + 1 === wordIndexes[1];

        if (punctuation) {
          tokens.push({
            text: punctuation,
            isMark: true
          });
        }

        tokens.push({
          text: token,
          isQuestionPart: isQuestionWord
        });

        if (hasDivider && divider) {
          tokens.push({
            text: divider,
            isQuestionPart: isSuccessive,
            isMark: true,
            isDivider: true
          });
        }

        if (isQuestionWord) {
          wordIndexes.shift();
        }

        currentWordIndex++;
        prevLastIndex = tokenRegExp.lastIndex;
      }

      const stopPunctuation = sentence.substring(prevLastIndex);
      tokens.push({
        text: stopPunctuation,
        isMark: true
      });

      log({ tokens });

      return tokens;
    }

    /**
     * Generate a question from a given phrase and source.
     * @param {string} phrase phrase to find.
     * @param {Source} src
     * @return {{ sentenceIndex: number, wordIndexes: Array<number> }}
     */

  }, {
    key: 'selectSentence',
    value: (() => {
      var ref = _asyncToGenerator(function* (_ref5) {
        let phrase = _ref5.phrase;
        let src = _ref5.src;

        const result = yield this.findSentenceIncludingPhrase({ phrase, src });

        if (!result) {
          return null;
        }

        const lemmatizedSentence = result.lemmatizedSentence;
        const sentenceIndex = result.index;

        const wordIndexes = this.findWordIndexesWithinLemmatizedSentence({
          lemmatizedSentence,
          phrase
        });

        log({
          phrase,
          src: _lodash2.default.last(src.path.split('/')),
          sentenceIndex,
          lemmatizedSentence,
          wordIndexes
        });

        return { sentenceIndex, wordIndexes };
      });

      function selectSentence(_x4) {
        return ref.apply(this, arguments);
      }

      return selectSentence;
    })()
  }, {
    key: 'findWordIndexesWithinLemmatizedSentence',
    value: function findWordIndexesWithinLemmatizedSentence(_ref6) {
      let lemmatizedSentence = _ref6.lemmatizedSentence;
      let phrase = _ref6.phrase;

      const sentence = lemmatizedSentence;
      const phraseRegExp = this.getPhraseRegExp(phrase);
      const wordIndexes = [];

      sentence.replace(phraseRegExp, (matched, block, offset) => {
        const left = sentence.substring(0, offset);
        const indexOffset = (left.match(this._wordRegExp) || []).length;
        const words = _lodash2.default.flatten(phrase.split('|').map(exp => exp.replace(this._abbrRegExp, '').match(this._wordRegExp) || []));
        let wordsCount = words.length;
        const escapedWords = words.map(word => word.replace(/\W/g, '\\$&')).join('|');
        const wordsRegExp = new RegExp(`(?:${ escapedWords })`, 'gi');

        block.replace(wordsRegExp, (word, offset2) => {
          if (wordsCount <= 0) {
            return;
          }

          const inlineLeft = block.substring(0, offset2);
          const inlineIndex = (inlineLeft.match(this._wordRegExp) || []).length;

          wordIndexes.push(indexOffset + inlineIndex);
          wordsCount--;
        });
      });

      return wordIndexes;
    }
  }, {
    key: 'findSentenceIncludingPhrase',
    value: (() => {
      var ref = _asyncToGenerator(function* (_ref7) {
        let src = _ref7.src;
        let phrase = _ref7.phrase;

        const text = yield src.lemmatized.getText();
        const phraseRegExp = this.getPhraseRegExp(phrase);
        const matches = [];

        // Find the phrase in the source text
        // and store the match information.
        // We use 'replace' here because nether RegExp#exec nor String#match
        // have a detail of each matches like a matched block and an offset etc.
        text.replace(phraseRegExp, function (matched, block, offset) {
          const lastIndex = offset + matched.length;
          matches.push({ matched, block, offset, lastIndex });
        });

        if (!matches.length) {
          return null;
        }

        log({
          phraseRegExp,
          src: _lodash2.default.last(src.path.split('/')),
          matches
        });

        const selected = _lodash2.default.sample(matches);
        const leftText = text.substring(0, selected.offset);
        const leftSentences = leftText.match(this._sentenceSeparator) || [];
        const leftContext = _lodash2.default.last(leftText.split(this._sentenceSeparator));
        const rightContext = _lodash2.default.head(text.substring(selected.lastIndex).split(this._sentenceSeparator));
        const lemmatizedSentence = leftContext + selected.matched + rightContext;
        const index = leftSentences.length;

        return { lemmatizedSentence, index };
      });

      function findSentenceIncludingPhrase(_x5) {
        return ref.apply(this, arguments);
      }

      return findSentenceIncludingPhrase;
    })()
  }, {
    key: 'getPhraseRegExp',
    value: function getPhraseRegExp(phrase) {
      const phraseWithAbbrRegExp = phrase.replace(this._abbrRegExp, `?(?:${ this._clauseRegExp.source })?`);
      const wordBoundaryRegExp = this._wordBoundaryRegExp.source;
      const phraseRegExp = new RegExp(`(?:^|${ wordBoundaryRegExp })(${ phraseWithAbbrRegExp })${ wordBoundaryRegExp }`, 'gim');

      return phraseRegExp;
    }

    /**
     * Get the 'index'-th sentence of the given text.
     * @param {number} index
     * @param {string} text
     * @return {string}
     */

  }, {
    key: 'getSentenceAt',
    value: function getSentenceAt(_ref8) {
      let index = _ref8.index;
      let text = _ref8.text;

      const body = text.split(this._sentenceSeparator)[index];

      /* istanbul ignore if */
      if (!body) {
        return null;
      }

      const rightContext = (text.split(body)[1] || '').substring(0, 5).trim();
      if (!rightContext) {
        return body;
      }
      const endMarks = rightContext.match(this._sentenceSeparator);
      if (!endMarks) {
        return body;
      }
      const endMark = endMarks[0].trim();

      return body + endMark;
    }
  }]);

  return QuizGenerator;
}();

exports.default = QuizGenerator;
//# sourceMappingURL=quiz-generator.js.map