'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fsExtraPromise = require('fs-extra-promise');

var _fsExtraPromise2 = _interopRequireDefault(_fsExtraPromise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

let Source = function () {

  /**
   * @param {string} path Path to a source file.
   * @param {boolean} shouldEnsureFile Whether or not create a file if not exists.
   */

  function Source(path, shouldEnsureFile) {
    _classCallCheck(this, Source);

    if (shouldEnsureFile) {
      _fsExtraPromise2.default.ensureFileSync(path);
    } else {
      _fsExtraPromise2.default.statSync(path);
    }

    this.path = path;
  }

  _createClass(Source, [{
    key: '_fetchContent',


    /**
     * Read the content of the file, parse it, and populate the results.
     */
    value: (() => {
      var ref = _asyncToGenerator(function* () {
        const content = yield _fsExtraPromise2.default.readFileAsync(this.path, 'utf-8');

        var _content$split = content.split(/[\n\r]/);

        var _content$split2 = _toArray(_content$split);

        const reference = _content$split2[0];

        const text = _content$split2.slice(1);

        this.setReference(reference);
        this.setText(text.join('\n'));
      });

      function _fetchContent() {
        return ref.apply(this, arguments);
      }

      return _fetchContent;
    })()

    /**
     * Return the reference information of this source.
     * @return {string}
     */

  }, {
    key: 'getReference',
    value: (() => {
      var ref = _asyncToGenerator(function* () {
        if (this._reference) {
          return this._reference;
        }

        yield this._fetchContent();
        return this._reference;
      });

      function getReference() {
        return ref.apply(this, arguments);
      }

      return getReference;
    })()

    /**
     * Set the reference information of this source.
     * @param {string} reference
     */

  }, {
    key: 'setReference',
    value: function setReference(reference) {
      this._reference = reference;
    }

    /**
     * Return the body text of this source.
     * @return {string}
     */

  }, {
    key: 'getText',
    value: (() => {
      var ref = _asyncToGenerator(function* () {
        if (this._text) {
          return this._text;
        }

        yield this._fetchContent();
        return this._text;
      });

      function getText() {
        return ref.apply(this, arguments);
      }

      return getText;
    })()

    /**
     * Set the body text of this source.
     * @param {string} text
     */

  }, {
    key: 'setText',
    value: function setText(text) {
      this._text = text;
    }

    /**
     * Write the reference and the body text to the file.
     */

  }, {
    key: 'save',
    value: (() => {
      var ref = _asyncToGenerator(function* () {
        const reference = yield this.getReference();
        const text = yield this.getText();
        yield _fsExtraPromise2.default.writeFileAsync(this.path, `${ reference }\n${ text }`);
      });

      function save() {
        return ref.apply(this, arguments);
      }

      return save;
    })()

    /**
     * Preprocess the text using the given preprocessor and save the result.
     * @param {Function<Promise<string>>} preprocessor
     */

  }, {
    key: 'preprocess',
    value: (() => {
      var ref = _asyncToGenerator(function* (preprocessor) {
        const source = new Source(`${ this.path }.preprocessed`, true);
        const reference = yield this.getReference();
        const text = yield this.getText();
        const preprocessed = yield preprocessor(text);

        source.setReference(reference);
        source.setText(preprocessed);
        yield source.save();
      });

      function preprocess(_x) {
        return ref.apply(this, arguments);
      }

      return preprocess;
    })()

    /**
     * Lemmatize the preprocessed text using the given lemmatizer
     * and save the result.
     * @param {Function<Promise<string>>} lemmatizer
     */

  }, {
    key: 'lemmatize',
    value: (() => {
      var ref = _asyncToGenerator(function* (lemmatizer) {
        const source = new Source(`${ this.path }.lemmatized`, true);
        const reference = yield this.preprocessed.getReference();
        const text = yield this.preprocessed.getText();
        const lemmatized = yield lemmatizer(text);

        source.setReference(reference);
        source.setText(lemmatized);
        yield source.save();
      });

      function lemmatize(_x2) {
        return ref.apply(this, arguments);
      }

      return lemmatize;
    })()
  }, {
    key: 'preprocessed',
    get: function get() {
      if (this._preprocessed) {
        return this._preprocessed;
      }

      try {
        return this._preprocessed = new Source(`${ this.path }.preprocessed`);
      } catch (err) {
        return this._preprocessed = this;
      }
    }
  }, {
    key: 'lemmatized',
    get: function get() {
      if (this._lemmatized) {
        return this._lemmatized;
      }

      try {
        return this._lemmatized = new Source(`${ this.path }.lemmatized`);
      } catch (err) {
        return this._lemmatized = this;
      }
    }
  }]);

  return Source;
}();

exports.default = Source;
//# sourceMappingURL=source.js.map